/*
 * JBoss, Home of Professional Open Source
 * Copyright 2009, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.remoting.tests;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServer;

import junit.framework.TestCase;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.jboss.remoting.Client;
import org.jboss.remoting.InvocationRequest;
import org.jboss.remoting.InvokerLocator;
import org.jboss.remoting.ServerInvocationHandler;
import org.jboss.remoting.ServerInvoker;
import org.jboss.remoting.callback.Callback;
import org.jboss.remoting.callback.HandleCallbackException;
import org.jboss.remoting.callback.InvokerCallbackHandler;
import org.jboss.remoting.marshal.MarshalFactory;
import org.jboss.remoting.marshal.UnMarshaller;
import org.jboss.remoting.marshal.http.HTTPUnMarshaller;
import org.jboss.remoting.transport.Connector;
import org.jboss.remoting.transport.PortUtil;
import org.jboss.remoting.transport.http.HTTPMetadataConstants;


public class HttpTest extends TestCase
{
   private static Logger log = Logger.getLogger(HttpTest.class);

   private static boolean firstTime = true;

   protected String host;
   protected int port;
   protected String locatorURI;
   protected InvokerLocator serverLocator;
   protected Connector connector;
   protected TestInvocationHandler invocationHandler;

   //   enum state {START, SAW_CR, SAW_EOL, SAW_2nd_CR};

   protected final int START = 1;
   protected final int SAW_CR = 2;
   protected final int SAW_EOL = 3;
   protected final int SAW_2nd_CR = 4;
   protected final int END_OF_HEADERS = 5;
   protected int State;

   public void setUp() throws Exception
   {
      if (firstTime)
      {
         firstTime = false;
         Logger.getLogger("org.jboss.remoting").setLevel(Level.INFO);
         Logger.getLogger("org.jboss.test.remoting").setLevel(Level.INFO);
         String pattern = "[%d{ABSOLUTE}] [%t] %5p (%F:%L) - %m%n";
         PatternLayout layout = new PatternLayout(pattern);
         ConsoleAppender consoleAppender = new ConsoleAppender(layout);
         Logger.getRootLogger().addAppender(consoleAppender);  
      }
   }


   public void tearDown()
   {
   }


   public void testMethod() throws Throwable
   {
      log.info("entering " + getName());

      // Start server.
      setupServer();

      // Create client.
      InvokerLocator clientLocator = new InvokerLocator("http://localhost:" + port);
      HashMap clientConfig = new HashMap();
      clientConfig.put(InvokerLocator.FORCE_REMOTE, "true");
      addExtraClientConfig(clientConfig);
      Client client = new Client(clientLocator, clientConfig);
      client.connect();
      log.info("client is connected");

      // Test connection.
      client.invokeOneway("abcdefghijklmnopqrstuvwxyz", null, true);
      Thread.sleep(300000);

      client.disconnect();
      shutdownServer();
      log.info(getName() + " PASSES");
   }


   protected String getTransport()
   {
      return "socket";
   }


   protected void addExtraClientConfig(Map config) {}
   protected void addExtraServerConfig(Map config) {}


   protected void setupServer() throws Exception
   {
      new Thread() 
      {
         public void run()
         {
            try
            {
               port = PortUtil.findFreePort(host);
               ServerSocket ss = new ServerSocket(port);
               Socket s = ss.accept();
               log.info("accepted " + s);
               InputStream is = s.getInputStream();
               DataInputStream dis = new DataInputStream(is);
               try
               {
                  //                        String st = dis.readUTF();
                  //                        System.out.println(st);
                  byte[] b = new byte[65536];
                  int c = 0;
                  State = START;
                  //                        int i = dis.read(b);
                  //                        System.out.println(i);
                  //                        System.out.println(new String(b));
                  while (true)
                  {
                     int i = dis.read();
                     log.info("read " + i);
                     //                        log.info(new String(b));

                     switch (State)
                     {
                        case START:
                           if (i == '\r')
                           {
                              State = SAW_CR;
                              log.info("entering SAW_CR");
                           }
                           else 
                           {
                              b[c++] = (byte) i;
                           }
                           break;

                        case SAW_CR:
                           if (i == '\n')
                           {
                              State = SAW_EOL;
                              log.info("entering SAW_EOL");
                           }
                           else
                           {
                              State = START;
                              log.info("returning to START");
                              b[c++] = (byte) '\r';
                              b[c++] = (byte) i;
                           }
                           break;

                        case SAW_EOL:
                           if (i == '\r')
                           {
                              State = SAW_2nd_CR;
                              log.info("entering SAW_2nd_CR");
                           }
                           else
                           {
                              State = START;
                              log.info("returning to START");
                              b[c++] = (byte) '\r';
                              b[c++] = (byte) '\n';
                              b[c++] = (byte) i;
                           }
                           break;

                        case SAW_2nd_CR:
                           if (i == '\n')
                           {
                              State = END_OF_HEADERS;
                              log.info("entering END_OF_HEADERS");
                           }
                           else
                           {
                              State = START;
                              log.info("returning to START");
                              b[c++] = (byte) '\r';
                              b[c++] = (byte) '\n';
                              b[c++] = (byte) '\r';
                              b[c++] = (byte) i; 
                           }
                           break;

                        default:
                           log.info("default: c == " + c);
                           throw new Exception("unexpected state: " + State);
                     }

                     if (State == END_OF_HEADERS)
                     {
                        log.info("c = " + c);
//                        for (int j = 0; j < c; j++) System.out.println(b[j]);
                        break;
                     }
                  }

                  log.info("finished with headers");
                  log.info(new String(b));
               } catch (EOFException e)
               {
                  System.out.println("=========");
               }
               
               UnMarshaller unmarshaller = new HTTPUnMarshaller(); 
               try
               {
                  HashMap metadata = new HashMap();
                  metadata.put("Content-Type", "application/octet-stream");
                  Object body = unmarshaller.read(dis, metadata);
                  log.info("body: " + body);
                  log.info("class: " + body.getClass());
               }
               catch (Throwable e)
               {
                  e.printStackTrace();
               }
               
               int i = dis.read();
               while (i != -1)
               {
                  System.out.println(i);
                  i = dis.read();
               }
               s.close();
               ss.close();
            } catch (Exception e)
            {
               e.printStackTrace();
            }
         }
      }.start();
   }


   protected void shutdownServer() throws Exception
   {
      if (connector != null)
         connector.stop();
   }


   static class TestInvocationHandler implements ServerInvocationHandler
   {
      public void addListener(InvokerCallbackHandler callbackHandler) {}
      public Object invoke(final InvocationRequest invocation) throws Throwable
      {
         return invocation.getParameter();
      }
      public void removeListener(InvokerCallbackHandler callbackHandler) {}
      public void setMBeanServer(MBeanServer server) {}
      public void setInvoker(ServerInvoker invoker) {}
   }


   static class TestCallbackHandler implements InvokerCallbackHandler
   {
      public void handleCallback(Callback callback) throws HandleCallbackException
      {
         log.info("received callback");
      }  
   }
}